/* Copyright (c) 2012, Steffen Wittmeier, Michael Jäntsch
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * * Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * * Neither the name of the Technische Universitaet Muenchen nor the names of
 *   its contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef _HAL_ECU_MUSCLE_H_
#define _HAL_ECU_MUSCLE_H_

#include "com.robot/Protocol.h"
#include "libmath/Types.h"
#include "libmath/Math.h"
#include "hal/libhal.h"
#include "Heartbeat.h"

#define MOTOR_POS0 (*(vu16*)0x0801FC04);
#define MOTOR_POS1 (*(vu16*)0x0801FC06);

extern bool reboot;

namespace eu
{
namespace er
{
namespace fw
{
namespace ecu
{
class Muscle;

typedef void
(Muscle::*MuscleMemFun)();

class Muscle
{
public:

	Muscle(u8 num, Heartbeat *hb);

	~Muscle();

private:

	void
	init(u8 * buffer);

public:

	void
	handleMsg(const u8 cmd, const u8 dlc, const u8 * data);

	void
	updateSensorData();

	void
	updateModel();

	void
	execControl();

	void
	heartbeatMissing();

	s32
	getMotorPosition();

	inline s16 getForce()
	{
		return force;
	}

	inline s16 getCurrent()
	{
		return current;
	}

	inline s16 getVoltage()
	{
		return motor.getVoltage();
	}

	inline void setSampleTime(const float t_sample)
	{
		t_s = t_sample;
	}

	inline com::robot::MuscleState getState()
	{
		return state;
	}

	void
	stateInitializeInit();
	void
	stateOffInit();
	void
	stateOnInit();

	void
	stateOffExecute();
	void
	stateOnExecute();
	void
	stateErrorExecute();

	inline u16 getCanID()
	{
		return canID;
	}

private:
	typedef struct
	{
		MuscleMemFun execute;

		MuscleMemFun init;
	} S_State;

	enum E_ForceControlMode
	{
		E_FCM_StateSpace, E_FCM_PD
	};

	u8 mynum;

	u8 canID;

	S_State states[com::robot::NumMuscleStates];

	volatile bool initialized;

	volatile u8 paramIndex;

	volatile com::robot::MuscleState state;
	com::robot::MuscleState lastState;

	volatile com::robot::E_ErrorCode errno;

	volatile com::robot::ControlVariable muscleControlMode;

	float t_s;

	hal::Encoder encoder;
	hal::Sensors sense;
	hal::Motor motor;
	hal::Can canBus;

	Heartbeat *hb;

	::el::math::Fixed encoderConversion;

        s16 force;
	s16 current;

	s32 maxMotorPosition;
	s32 minMotorPosition;

	s32 maxForce;

	s16 currentOffset;

	s16 forceSign;
	s16 forceOffset;
	::el::math::Fixed forceGain;

	s16 maxPosVoltage;
	s16 minPosVoltage;
	s16 maxNegVoltage;
	s16 minNegVoltage;

	u16 pMotorPosition;
	u16 epsPoti;
	el::math::Fixed forceCompensation;

	::el::math::Fixed refValue;
};
}
}
}
}

#endif

