/* Copyright (c) 2012, Steffen Wittmeier, Michael Jäntsch
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * * Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * * Neither the name of the Technische Universitaet Muenchen nor the names of
 *   its contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include "Can.h"
#include "com.robot/Protocol.h"

namespace eu
{
namespace er
{
namespace fw
{
namespace ecu
{
namespace hal
{
bool Can::caninitialized = false;

bool Can::receiveBroadcast = true;

void
(*Can::can_interrupt_handler)(void) = 0;

Can::Can()
{
	if (!caninitialized)
	{
		initializeCan();
	}
}

///One-time hardware initialization routine.
void Can::initializeCan()
{
	CAN_InitTypeDef CAN_InitStructure;
	CAN_FilterInitTypeDef CAN_FilterInitStructure;

	GPIO_InitTypeDef GPIO_InitStructure;
	NVIC_InitTypeDef NVIC_InitStructure;

	/* enable CAN IRQ */
	NVIC_InitStructure.NVIC_IRQChannel = USB_LP_CAN_RX0_IRQChannel;
	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 1;
	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&NVIC_InitStructure);

	/* GPIOA clock enable */
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA, ENABLE);
	/* CAN Periph clock enable */
	RCC_APB1PeriphClockCmd(RCC_APB1Periph_CAN, ENABLE);

	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;

	/* Configure CAN pin: RX */
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_11;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IPU;
	GPIO_Init(GPIOA, &GPIO_InitStructure);

	/* Configure CAN pin: TX */
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_12;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
	GPIO_Init(GPIOA, &GPIO_InitStructure);

	/* CAN register init */
	CAN_DeInit();
	CAN_StructInit(&CAN_InitStructure);

	/* CAN cell init */
	CAN_InitStructure.CAN_TTCM = DISABLE;
	CAN_InitStructure.CAN_ABOM = DISABLE;
	CAN_InitStructure.CAN_AWUM = DISABLE;
	CAN_InitStructure.CAN_NART = DISABLE;
	CAN_InitStructure.CAN_RFLM = DISABLE;
	CAN_InitStructure.CAN_TXFP = DISABLE;
	CAN_InitStructure.CAN_Mode = CAN_Mode_Normal;
	/*CAN_InitStructure.CAN_SJW=CAN_SJW_1tq;
	 CAN_InitStructure.CAN_BS1=CAN_BS1_8tq;
	 CAN_InitStructure.CAN_BS2=CAN_BS2_7tq;*/
	CAN_InitStructure.CAN_BS1 = CAN_BS1_12tq;
	CAN_InitStructure.CAN_BS2 = CAN_BS2_5tq;
	CAN_InitStructure.CAN_Prescaler = 2;
	CAN_Init(&CAN_InitStructure);

	u16 canid0 = CAN_ID0;
	u16 canid1 = CAN_ID1;
	u16 canid2 = CAN_ID_JOINT;

	/* CAN filter init */
	CAN_FilterInitStructure.CAN_FilterNumber = 0;
	CAN_FilterInitStructure.CAN_FilterMode = CAN_FilterMode_IdMask;
	CAN_FilterInitStructure.CAN_FilterScale = CAN_FilterScale_16bit;
	//Shift id 5 to left because of extended id, rtr, ide bits
	CAN_FilterInitStructure.CAN_FilterIdHigh = canid0 << 5;
	CAN_FilterInitStructure.CAN_FilterIdLow = canid1 << 5;
	CAN_FilterInitStructure.CAN_FilterMaskIdHigh
			= com::robot::CAN_CONTROLLER_ID_MASK << 5;
	CAN_FilterInitStructure.CAN_FilterMaskIdLow
			= com::robot::CAN_CONTROLLER_ID_MASK << 5;
	CAN_FilterInitStructure.CAN_FilterFIFOAssignment = CAN_FIFO0;
	CAN_FilterInitStructure.CAN_FilterActivation = ENABLE;
	CAN_FilterInit(&CAN_FilterInitStructure);

	if (receiveBroadcast)
	{
		/* CAN broadcast filter init */
		CAN_FilterInitStructure.CAN_FilterNumber = 1;
		CAN_FilterInitStructure.CAN_FilterMode = CAN_FilterMode_IdMask;
		CAN_FilterInitStructure.CAN_FilterScale = CAN_FilterScale_16bit;
		CAN_FilterInitStructure.CAN_FilterIdHigh
				= com::robot::BROADCAST_CONTROLLER_ID << 5; //Broadcast
		CAN_FilterInitStructure.CAN_FilterIdLow = 0; //unused
		CAN_FilterInitStructure.CAN_FilterMaskIdHigh
				= com::robot::CAN_CONTROLLER_ID_MASK << 5;
		CAN_FilterInitStructure.CAN_FilterMaskIdLow = 0xFFFF; //unused
		CAN_FilterInitStructure.CAN_FilterFIFOAssignment = CAN_FIFO0;
		CAN_FilterInitStructure.CAN_FilterActivation = ENABLE;
		CAN_FilterInit(&CAN_FilterInitStructure);
	}

	CAN_FilterInitStructure.CAN_FilterNumber = 2;
	CAN_FilterInitStructure.CAN_FilterMode = CAN_FilterMode_IdMask;
	CAN_FilterInitStructure.CAN_FilterScale = CAN_FilterScale_16bit;
	//Shift id 5 to left because of extended id, rtr, ide bits
	CAN_FilterInitStructure.CAN_FilterIdHigh = canid2 << 5;
	CAN_FilterInitStructure.CAN_FilterIdLow = 0; //unused
	CAN_FilterInitStructure.CAN_FilterMaskIdHigh
			= com::robot::CAN_CONTROLLER_ID_MASK << 5;
	CAN_FilterInitStructure.CAN_FilterMaskIdLow
			= com::robot::CAN_CONTROLLER_ID_MASK << 5;
	CAN_FilterInitStructure.CAN_FilterFIFOAssignment = CAN_FIFO0;
	CAN_FilterInitStructure.CAN_FilterActivation = ENABLE;
	CAN_FilterInit(&CAN_FilterInitStructure);

	/* CAN FIFO0 message pending interrupt enable */
	CAN_ITConfig(CAN_IT_FMP0, ENABLE);

	/* set caninitialized (needs to run only once) */
	caninitialized = true;
}

void Can::setReceiveBroadcast(bool receiveBroadcast)
{
	Can::receiveBroadcast = receiveBroadcast;

	//TODO initialize filter here
}

///Switch CAN controller on.
void Can::on()
{
	/* can is completely re-initialized to clear old messages */
	initializeCan();
	CAN_WakeUp();
}

///Switch CAN controller off.
void Can::off()
{
	/* disable interrupt handling */
	CAN_ITConfig(CAN_IT_FMP0, DISABLE);
}

s8 Can::send(u32 can_id, u8 dlc, u8 d0, u8 d1, u8 d2, u8 d3, u8 d4, u8 d5,
		u8 d6, u8 d7, u8 ide, u8 rtr)
{
	CanTxMsg txMsg;

	txMsg.RTR = rtr;
	txMsg.IDE = ide;
	txMsg.DLC = dlc;

	txMsg.Data[0] = d0;
	txMsg.Data[1] = d1;
	txMsg.Data[2] = d2;
	txMsg.Data[3] = d3;
	txMsg.Data[4] = d4;
	txMsg.Data[5] = d5;
	txMsg.Data[6] = d6;
	txMsg.Data[7] = d7;

	if (CAN_ID_STD == ide)
		txMsg.StdId = (u16) can_id;
	else
		txMsg.ExtId = can_id;

	return send(&txMsg);
}

/**
 * @brief Send a CAN message.
 * @param TxMessage Message to send.
 */
s8 Can::send(CanTxMsg *TxMessage)
{
	//  /* debug code */
	//  /*TODO remove debug*/
	//  ret = 0;

	/* send TxMessage */
	u8 TransmitMailbox = CAN_Transmit(TxMessage);
	//Error: Message has not been transmitted
	if (!IS_CAN_TRANSMITMAILBOX(TransmitMailbox))
	{
		return -1;
	}

	return 0;
}

/**
 * @brief Receive a CAN message.
 * @param RxMessage Structure where the received message will be stored.
 */
void Can::receive(CanRxMsg *RxMessage)
{
	CAN_Receive(CAN_FIFO0, RxMessage);
}

/**
 * @brief Set a CAN interrupt handler.
 *
 * Be really careful that your function here does not use loops, many function calls
 * or any things that rely on interrupts or block, as the handler will be executed in
 * interrupt context.
 *
 * @param canInterruptHandler The interrupt handler to set.
 */
void Can::setCanInterruptHandler(void(*interrupt_handler)(void))
{
	can_interrupt_handler = interrupt_handler;
}

}
}
}
}
}

#ifndef TIMING_CAN_HANDLER

extern "C"
{
///CAN interrupt handler.
void __attribute__((__interrupt__))
USB_LP_CAN_RX0_IRQHandler(void)
{
	if (::eu::er::fw::ecu::hal::Can::can_interrupt_handler != 0)
		eu::er::fw::ecu::hal::Can::can_interrupt_handler();
}

} //extern "C"

#endif
