/* Copyright (c) 2012, Steffen Wittmeier, Michael Jäntsch
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * * Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * * Neither the name of the Technische Universitaet Muenchen nor the names of
 *   its contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#define _TIM4

//#include <stdlib.h>

#include "IntervalTimer.h"
#include "libstm32/stm32fwlib.h"

#ifdef DEBUG
TIM_TypeDef * TIM3 = (TIM_TypeDef *) TIM3_BASE;
TIM_TypeDef * TIM4 = (TIM_TypeDef *) TIM4_BASE;
#endif

namespace eu
{
namespace er
{
namespace fw
{
namespace ecu
{
namespace hal
{

IntervalTimer::TimerHandler IntervalTimer::m_timerHandler[] =
{ 0, 0, 0, 0 };
bool IntervalTimer::initialized = false;
u16 IntervalTimer::m_timerInterval[] =
{ (u16) 0, (u16) 0, (u16) 0, (u16) 0 };

IntervalTimer::IntervalTimer(E_Timer tim) :
	m_timer(tim)
{
	if (!initialized)
	{
		init();
	}
}

void IntervalTimer::init(void)
{
	TIM_TimeBaseInitTypeDef TIM_TimeBaseStructure;
	NVIC_InitTypeDef NVIC_InitStructure;

	//enable clock
	RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM4, ENABLE);

	/* Enable the TIM4 global Interrupt */
	NVIC_InitStructure.NVIC_IRQChannel = TIM4_IRQChannel;
	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 3;
	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 1;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;

	NVIC_Init(&NVIC_InitStructure);

	//Default period is 2ms
	TIM_TimeBaseStructure.TIM_Period = 65535;
	TIM_TimeBaseStructure.TIM_Prescaler = 0;
	TIM_TimeBaseStructure.TIM_ClockDivision = TIM_CKD_DIV1;
	TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;
	TIM_TimeBaseInit(TIM4, &TIM_TimeBaseStructure);

	//Prescaler configuration 72MHz / 7200 = 10 kHz
	TIM_PrescalerConfig(TIM4, 7200, TIM_PSCReloadMode_Immediate);

	// Output Compare Timing Mode configuration: Channel1
	TIM_OCInitTypeDef TIM_OCInitStructure;
	TIM_OCInitStructure.TIM_OCMode = TIM_OCMode_Timing;
	TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Enable;
	TIM_OCInitStructure.TIM_Pulse = 2000;
	TIM_OCInitStructure.TIM_OCPolarity = TIM_OCPolarity_High;
	TIM_OC1Init(TIM4, &TIM_OCInitStructure);
	TIM_OC1PreloadConfig(TIM4, TIM_OCPreload_Disable);

	TIM_OCInitStructure.TIM_Pulse = 2000;
	TIM_OC2Init(TIM4, &TIM_OCInitStructure);
	TIM_OC2PreloadConfig(TIM4, TIM_OCPreload_Disable);

	TIM_OCInitStructure.TIM_Pulse = 2000;
	TIM_OC3Init(TIM4, &TIM_OCInitStructure);
	TIM_OC3PreloadConfig(TIM4, TIM_OCPreload_Disable);

	TIM_OCInitStructure.TIM_Pulse = 2000;
	TIM_OC4Init(TIM4, &TIM_OCInitStructure);
	TIM_OC4PreloadConfig(TIM4, TIM_OCPreload_Disable);

	TIM_Cmd(TIM4, ENABLE);

	initialized = true;
}

void IntervalTimer::enable(void)
{
	switch (m_timer)
	{
	case E_TIMER1:
		TIM_ClearITPendingBit(TIM4, TIM_IT_CC1);
		TIM_ITConfig(TIM4, TIM_IT_CC1, ENABLE);
		break;
	case E_TIMER2:
		TIM_ClearITPendingBit(TIM4, TIM_IT_CC2);
		TIM_ITConfig(TIM4, TIM_IT_CC2, ENABLE);
		break;
	case E_TIMER3:
		TIM_ClearITPendingBit(TIM4, TIM_IT_CC3);
		TIM_ITConfig(TIM4, TIM_IT_CC3, ENABLE);
		break;
	case E_TIMER4:
		TIM_ClearITPendingBit(TIM4, TIM_IT_CC4);
		TIM_ITConfig(TIM4, TIM_IT_CC4, ENABLE);
		break;
	}
}

u16 IntervalTimer::getValue()
{
	return TIM_GetCounter(TIM4);
}

void IntervalTimer::disable(void)
{
	switch (m_timer)
	{
	case E_TIMER1:
		TIM_ITConfig(TIM4, TIM_IT_CC1, DISABLE);
		break;
	case E_TIMER2:
		TIM_ITConfig(TIM4, TIM_IT_CC2, DISABLE);
		break;
	case E_TIMER3:
		TIM_ITConfig(TIM4, TIM_IT_CC3, DISABLE);
		break;
	case E_TIMER4:
		TIM_ITConfig(TIM4, TIM_IT_CC4, DISABLE);
		break;
	}
}

bool IntervalTimer::isEnabled()
{

	switch (m_timer)
	{
	case E_TIMER1:
		return (TIM4->DIER & TIM_IT_CC1) != 0;
	case E_TIMER2:
		return (TIM4->DIER & TIM_IT_CC2) != 0;
	case E_TIMER3:
		return (TIM4->DIER & TIM_IT_CC3) != 0;
	case E_TIMER4:
		return (TIM4->DIER & TIM_IT_CC4) != 0;
	}
}

void IntervalTimer::setInterval(const u16 tenth_milliseconds)
{
	m_timerInterval[m_timer] = tenth_milliseconds;

	switch (m_timer)
	{
	case E_TIMER1:
		TIM_SetCompare1(TIM4, getValue() + tenth_milliseconds);
		break;
	case E_TIMER2:
		TIM_SetCompare2(TIM4, getValue() + tenth_milliseconds);
		break;
	case E_TIMER3:
		TIM_SetCompare3(TIM4, getValue() + tenth_milliseconds);
		break;
	case E_TIMER4:
		TIM_SetCompare4(TIM4, getValue() + tenth_milliseconds);
		break;
	}
}

void IntervalTimer::setHandler(void(*handler)(void))
{
	m_timerHandler[m_timer] = handler;
}

}
}
}
}
}

extern "C"
{
void __attribute__((__interrupt__))
TIM4_IRQHandler(void)
{
	if (TIM_GetITStatus(TIM4, TIM_IT_CC1))
	{
		TIM_ClearITPendingBit(TIM4, TIM_IT_CC1);
		TIM_SetCompare1(
				TIM4,
				TIM_GetCapture1(TIM4)
						+ ::eu::er::fw::ecu::hal::IntervalTimer::m_timerInterval[::eu::er::fw::ecu::hal::IntervalTimer::E_TIMER1]);

		if (::eu::er::fw::ecu::hal::IntervalTimer::m_timerHandler[::eu::er::fw::ecu::hal::IntervalTimer::E_TIMER1]
				!= 0)
			::eu::er::fw::ecu::hal::IntervalTimer::m_timerHandler[::eu::er::fw::ecu::hal::IntervalTimer::E_TIMER1]();
	}
	else if (TIM_GetITStatus(TIM4, TIM_IT_CC2))
	{
		TIM_ClearITPendingBit(TIM4, TIM_IT_CC2);
		TIM_SetCompare2(
				TIM4,
				TIM_GetCapture2(TIM4)
						+ ::eu::er::fw::ecu::hal::IntervalTimer::m_timerInterval[::eu::er::fw::ecu::hal::IntervalTimer::E_TIMER2]);

		if (::eu::er::fw::ecu::hal::IntervalTimer::m_timerHandler[::eu::er::fw::ecu::hal::IntervalTimer::E_TIMER2]
				!= 0)
			::eu::er::fw::ecu::hal::IntervalTimer::m_timerHandler[::eu::er::fw::ecu::hal::IntervalTimer::E_TIMER2];
	}
	else if (TIM_GetITStatus(TIM4, TIM_IT_CC3))
	{
		TIM_ClearITPendingBit(TIM4, TIM_IT_CC3);
		TIM_SetCompare3(
				TIM4,
				TIM_GetCapture3(TIM4)
						+ ::eu::er::fw::ecu::hal::IntervalTimer::m_timerInterval[::eu::er::fw::ecu::hal::IntervalTimer::E_TIMER3]);

		if (::eu::er::fw::ecu::hal::IntervalTimer::m_timerHandler[::eu::er::fw::ecu::hal::IntervalTimer::E_TIMER3]
				!= 0)
			::eu::er::fw::ecu::hal::IntervalTimer::m_timerHandler[::eu::er::fw::ecu::hal::IntervalTimer::E_TIMER3]();
	}
	else if (TIM_GetITStatus(TIM4, TIM_IT_CC4))
	{
		TIM_ClearITPendingBit(TIM4, TIM_IT_CC4);
		TIM_SetCompare4(
				TIM4,
				TIM_GetCapture4(TIM4)
						+ ::eu::er::fw::ecu::hal::IntervalTimer::m_timerInterval[::eu::er::fw::ecu::hal::IntervalTimer::E_TIMER4]);

		if (::eu::er::fw::ecu::hal::IntervalTimer::m_timerHandler[::eu::er::fw::ecu::hal::IntervalTimer::E_TIMER4]
				!= 0)
			::eu::er::fw::ecu::hal::IntervalTimer::m_timerHandler[::eu::er::fw::ecu::hal::IntervalTimer::E_TIMER4]();
	}
}
}
