/* Copyright (c) 2012, Steffen Wittmeier, Michael Jäntsch
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * * Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * * Neither the name of the Technische Universitaet Muenchen nor the names of
 *   its contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include "Motor.h"

#include "libhal.h"

namespace eu
{
namespace er
{
namespace fw
{
namespace ecu
{
namespace hal
{

Motor::Motor()
{
	GPIO_InitTypeDef GPIO_InitStructure;
	TIM_TimeBaseInitTypeDef TIM_TimeBaseStructure;

	/* TIM2 clock enable */
	RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM2, ENABLE);

	/* GPIOB clock enable */
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOB, ENABLE);
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOC, ENABLE);
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_AFIO, ENABLE);

	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_10MHz;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_8;
	GPIO_Init(GPIOB, &GPIO_InitStructure);

	/* Pin Remap for Channel 3 + 4 of TIM 2 -> PB10 + PB11 */
	GPIO_PinRemapConfig(GPIO_PartialRemap2_TIM2, ENABLE);

	/*GPIOB Configuration: TIM2 channel 3 and 4 (PB10; PB11) as alternate function push-pull */
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_10 | GPIO_Pin_11;

	GPIO_Init(GPIOB, &GPIO_InitStructure);

	// initialize HS-ON Pin for H-Bridges
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_10;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_10MHz;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
	GPIO_Init(GPIOC, &GPIO_InitStructure);

	/* Timer base configuration, set up timer to run with 50kHz */
	//TODO: use TIM_TimeBaseInit

	TIM_ARRPreloadConfig(TIM2, ENABLE);

	/* TIM2 enable counter */
	TIM_Cmd(TIM2, ENABLE);

	// set HS-ON Pin for H-Bridges high
	GPIO_SetBits(GPIOC, GPIO_Pin_10);

	/* init PWM for specific channel */
	//TODO: use TIM_OC3PreloadConfig
}

void Motor::setVoltage(s32 voltage)
{
	/* set duty cycle on PWM relative to reference voltage (macro in Motor.h: REFVOLTAGE) */
	//TODO: use TIM_OC3Init

	/* set motor direction control pin (GPIOB, GPIO_Pin_8), depending on voltage sign */
	//TODO: GPIO_SetBits
}

s32 Motor::getVoltage()
{
	/* get duty cycle on PWM relative to reference voltage (macro in Motor.h: REFVOLTAGE) */
	//TODO

	/* get voltage sign, depending on motor direction control pin (GPIOB, GPIO_Pin_8) */
	//TODO

}

void Motor::on()
{
	TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Enable;

	TIM_OC3Init(TIM2, &TIM_OCInitStructure);
}

void Motor::off()
{
	TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Disable;

	TIM_OC3Init(TIM2, &TIM_OCInitStructure);
}

bool Motor::isOn(void)
{
	return (TIM_OCInitStructure.TIM_OutputState == TIM_OutputState_Enable);
}

}
}
}
}
}
