/* Copyright (c) 2012, Steffen Wittmeier, Michael Jäntsch
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * * Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * * Neither the name of the Technische Universitaet Muenchen nor the names of
 *   its contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef SENSORS_H_
#define SENSORS_H_

#include "libstm32/stm32fwlib.h"

#define ADC1_DR_Address    ((u32)0x4001244C)

namespace eu
{
namespace er
{
namespace fw
{
namespace ecu
{
namespace hal
{

/**
 * @brief Encapsulates the sensors driver for sensors attached to the ADC.
 *
 */
class Sensors
{

public:
	enum E_Sensor
	{
		E_SENS_Position = 0,
		E_SENS_Force = 1,
		E_SENS_Unamp_Force = 2,
		E_SENS_Current = 3,
		E_SENS_Recoil = 4
	};

	Sensors();

	void
	startConv();

	/**
	 * @brief Check if the conversion has finished.
	 * @return <b>true</b> if the conversion has finished, or
	 * 	<b>false</b> if it has not finished.
	 */
	bool convFinished()
	{
		return DMA_GetFlagStatus(DMA1_FLAG_TC1);
	}

	u16 getPosition(u8 muscle)
	{
		return static_cast<u16> ((static_cast<u32> (getValue(E_SENS_Position,
				muscle)) * 3600) / 4095);
	}

	s16 getCurrent(u8 muscle)
	{
		//return static_cast<s16> ((static_cast<s32> (getValue(E_SENS_Current,
		//muscle)) * 121 - 250000) / 10);
		return static_cast<s16> ((static_cast<s32> (getValue(E_SENS_Current,
				muscle)) * 1281) / 100);
	}

	u16 getForce(u8 muscle)
	{
		return getValue(E_SENS_Force, muscle);
		//return static_cast< u16 > ((static_cast< u32 > (getValue(E_SENS_Strain, muscle)) * 1000) / 117);
	}

	u16 getUnamplifiedForce(u8 muscle)
	{
		return getValue(E_SENS_Unamp_Force, muscle);
	}

	u16 getRecoilLength(u8 muscle)
	{
		return static_cast<u16> ((static_cast<u32> (getValue(E_SENS_Recoil,
				muscle)) * 3600) / 4095);
	}

	u16 getFSR1()
	{
		return getValue(E_SENS_Position, 0);
	}

	u16 getFSR2()
	{
		return getValue(E_SENS_Position, 1);
	}

	u16 getFSR3()
	{
		return getValue(E_SENS_Unamp_Force, 0);
	}

private:

	u16
	getValue(E_Sensor sensor, u8 muscle);

	static void
	initializeADC();

	static bool adcinitialized;

	static vu32 dmabuf[5];

};

}
}
}
}
}
#endif
