/* Copyright (c) 2012, Steffen Wittmeier, Michael Jäntsch
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * * Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * * Neither the name of the Technische Universitaet Muenchen nor the names of
 *   its contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef SYS_TIMER_H_
#define SYS_TIMER_H_

#include "libstm32/stm32fwlib.h"
#include "Motor.h"

namespace eu
{
namespace er
{
namespace fw
{
namespace ecu
{
namespace hal
{

/**
 * @brief A timer driver which permits the user program to block
 * execution for a certain time.
 *
 * The timer can be started at the beginning of a computation. When
 * the computatation is finished, the <code>wait</code>-method is called,
 * which delays the execution until the timer has expired. This is useful
 * to guarantee a certain loop frequency for the control loop.<br/>
 * If the control loop code needs more time than the timer is programmed
 * for, an error flag is set which indicates this error.
 */
class SysTimer
{
public:

	struct Time
	{
		u32 microSeconds;
		u32 seconds;
	};

	static void
	start(u16 period);

	static void
	stop();

	static void
	wait();

	static u32
	getValue();

	static void getTime(Time &t)
	{
		t.microSeconds = SysTimer::t.microSeconds + SysTick_GetCounter() / 9;
		t.seconds = SysTimer::t.seconds;
	}

	static u16 period;

	static vu8 timer_flag;

	static volatile Time t;

};
}
}
}
}
}

#endif
